import json
import subprocess

from agents.Agent import Agent
from agents.Config import ModelConfig
from agents.CleanOutput import CleanOutput

def terminalUse(task: str, debug: bool = False) -> str:
    _taskAgent = Agent(
        name="_agent",
        instruction="You need to break the task in steps that can be done in 1 command.",
    )
    _commandAgent = Agent(
        name="commandAgent",
        instruction="From the given steps you need to give me commands",
    )

    promptTaskAgent = f"""
        "You are now an AI agent.

        Agent information:
            - Agent name: {_taskAgent.name}
            - Agent instruction: {_taskAgent.instruction}
            - Task: {task}

        The above list defines you. You can't make any other info up.
        
        You to break down the task into steps that can be done in 1 command.
            
        Follow these instructions precisely."
    """
    
    response = _taskAgent.run(prompt=promptTaskAgent, debug=debug)

    promptCommandAgent = f"""
        "You are now an AI agent.

        Agent information:
            - Agent name: {_commandAgent.name}
            - Agent instruction: {_commandAgent.instruction}
            - Steps: {response}

        The above list defines you. You can't make any other info up.
        
        You need to give commands to fulfill the steps.
        
        You need to give an output like this (valid JSON):

        {{
            "commands": [
                {{
                    "id": "1",
                    "command": "{{enter command here}}"
                }},
                {{
                    "id": "2",
                    "command": "{{enter command here}}"
                }}
            ]
        }}

        Extra instructions:
            - You need to only generate command for the given steps. So don't add unnecessary steps. 
            - Only Respond with valid json. Don't add anything else so no: '```json'
            
        Follow these instructions precisely."
    """
    
    taskResponse = _commandAgent.run(prompt=promptCommandAgent, debug=debug)
    taskResponse = CleanOutput(stdout=taskResponse, openAI=ModelConfig.getDefaultOpenAI(), debug=debug)

    try:
        commandsJson = json.loads(taskResponse)
        for cmd in commandsJson.get("commands", []):
            print(f"\nRunning command [{cmd['id']}]: {cmd['command']}")
            result = subprocess.run(cmd["command"], shell=True, capture_output=True, text=True)
            print(f"Output:\n{result.stdout}")
            if result.stderr:
                print(f"Errors:\n{result.stderr}")
    except json.JSONDecodeError:
        print("Failed to parse command output as JSON")
    except Exception as e:
        print(f"Unexpected error: {e}")

if __name__ == "__main__":
    ModelConfig.setDefaultModel("gpt-4o", True)
    terminalUse("Scan netwerk op actieve hosts", debug=True)



import json

from agents.Agent import Agent
from agents.Config import ModelConfig
from agents.CleanOutput import CleanOutput

class Chain:
    def __init__(self, agents: list['Agent']):
        self.agents = agents
    
    def execute(self, prompt: str, disableGuardrails: bool = False, debug: bool = False) -> dict:
        results = {}
        currentPrompt = prompt
        
        if debug:
            print(f"[DEBUG] agents in crew: {self.agents}")

        for agent in self.agents:
            result = agent.run(currentPrompt, debug, disableGuardrails)
            results[agent.name] = result
            
            if debug:
                print(f"[DEBUG] Results {agent.name}: {result}")
                
            currentPrompt += f"\n{agent.name} response: {result}"

        return results




    def runUntil(self, prompt: str, exitValue: str, maxRuns: int = 0, disableGuardrails: bool = False, debug: bool = False) -> str:
        results = {}
        currentPrompt = prompt

        maxRuns = maxRuns * len(self.agents)

        _agent = Agent(
            name="agent",
            instruction="You need to decide if the exit conditions are met.",
            model=ModelConfig.getDefaultModel(),
            openAI=ModelConfig.getDefaultOpenAI(),
        )

        if debug:
            print(f"[DEBUG] _agent.openAI: {_agent.openAI}")
            print(f"[DEBUG] agents in crew: {self.agents}")
            print(f"[DEBUG] exitCondition: {exitValue}")
        
        i = 0

        while i < maxRuns:
            for agent in self.agents:
                result = agent.run(currentPrompt, debug, disableGuardrails)
                results[agent.name] = result

                if debug:
                    print(f"[DEBUG] Results {agent.name}: {result}")

                currentPrompt += f"\n{agent.name} response: {result}"

            promptRunUntilExitValue = f"""
                "You are now an AI agent.

                    Agent information:
                        - Agent instruction: {_agent.instruction}
                        - Prompt: {prompt}
                        - The conversation: {currentPrompt}
                        - The exit condition: {exitValue}

                    The above list defines you. You can't make any other info up.

                    You need to decide if the exit conditions have been met. If so, respond with a JSON like this:

                    {{
                        "exitCondition": "yes", # make it no if it hasn't been met.
                        "goodAnswerToPrompt": "The answer to 4+1=5" # leave empty if exit condition is no
                    }}

                    Extra instructions:
                        - Only respond with valid JSON. Don't add anything else.
                        - Never use codeblocks (```json ... ```)
                "
            """

            if _agent.openAI:
                stdout = _agent.runOpenAI(promptRunUntilExitValue, debug=debug)
            else:
                stdout = _agent.runLocalModel(promptRunUntilExitValue, debug=debug)

            data = CleanOutput(stdout, ModelConfig.getDefaultOpenAI(), debug=debug)

            if debug:
                print(f"[DEBUG] JSON output from decision agent: {data}")

            i += 1
            
            try:
                if data.get("exitCondition") == "yes":
                    return data.get("goodAnswerToPrompt", "")
            except Exception as e:
                if debug:
                    print(f"[DEBUG] Failed to parse JSON output: {e}")
                    print(f"[DEBUG] Raw output: {data}")

        return results


import inspect
import re
import json
import functools

from functools import wraps

from agents.Agent import Agent

defaultModel = "gpt-4o"
defaultOpenAI = True

debug = False

def dynamicTool(function):
    @functools.wraps(function)
    def wrapper(*args, prompt=None, **kwargs):
        sig = inspect.signature(function)

        try:
            bound_args = sig.bind(*args, **kwargs)
            bound_args.apply_defaults()
            return function(*args, **kwargs)
        
        except TypeError as e:
            if debug:
                print(f"[DEBUG] Argument mismatch, AI gaat invullen: {e}")

        source = inspect.getsource(function)
        param_list = list(sig.parameters.keys())

        dynamicAgent = Agent(
            name="dynamicAgent",
            instruction="",
            model=defaultModel,
            openAI=defaultOpenAI,
            outputGuardrails="You need to check if the given parameters will not cause problems, so dont't use spaces in links. Or other erros that can be prevented."
        )

        prompt_for_params = f"""
            You are now an AI agent.

            Agent information:
                - Agent name: {dynamicAgent.name}
                - Agent instruction: {dynamicAgent.instruction}
                - Function parameters: {param_list}
                - Function code: {source}
                - User prompt: {prompt}

            The above list defines you. You can't make any other info up.

            You need to generate the right parameters for the function. So the right output will be given. If the name is somehting like city then don't give a country as parameter. And other stuff like that.

            Respond ONLY with valid JSON like:

            {{
                "parameters": {{
                    "parameter1": "value1",
                    "parameter2": "value2"
                }}
            }}
        """

        stdout = dynamicAgent.runOpenAI(prompt_for_params, debug=debug)
        rawOutput = stdout.choices[0].message.content.strip()

        clean_output = re.sub(r"^```(?:json)?\n|\n```$", "", rawOutput)

        try:
            data = json.loads(clean_output)
        except json.JSONDecodeError as e:
            print(f"[ERROR] Failed to parse JSON: {e}")
            return None

        if "parameters" not in data:
            print("[ERROR] JSON is missing 'parameters' key.")
            return None

        return function(**data["parameters"])

    wrapper.__dynamic_tool__ = True # Add for the toolcheck thingy 
    return wrapper



@dynamicTool
def _getWeatherData(city: str):
    if city.lower() == "london":
        return "London: sun"
    elif city.lower() == "singapore":
        return "Singapore: amazing weather"
    elif city.lower() == "washington dc":
        return "Washington DC: thunder"
    else:
        return f"{city}: no weather data found."

agent = Agent(
    name="agent",
    instruction="You need to run the tools based on the prompt",
    model="gpt-4o",
    openAI=True,
    tools=[_getWeatherData]
)


if __name__ == "__main__":
    r = agent.run(input("prompt: "), debug=debug, disableGuardrails=False)
    print(r)


import re
import json

def CleanOutput(stdout: str, openAI: bool, debug: bool = False):
    
    rawOutput = stdout.choices[0].message.content.strip()

    clean_output = re.sub(r"^```(?:json)?\n|\n```$", "", rawOutput)

    data = json.loads(clean_output)

    try:
        if openAI:
            data = json.loads(stdout.choices[0].message.content.strip())
        elif not openAI:
            data = json.loads(stdout.strip())
        else:
            data = "This should never happen..."
        
        if debug:
            print(f"[DEBUG] openAI: {openAI}")
            print(f"[DEBUG] data: {data}")
        
        return data
        
    except json.JSONDecodeError as e:
        if debug:
            print(f"[ERROR] Failed to decode JSON: {e}")
        return None


from Agent import Agent
from Chain import Chain
from Config import ModelConfig

ModelConfig.setDefaultModel("gpt-4o", True)

john = Agent(
    name="john",
    instruction="You are john, you need to figure out a solution based on the problem.",
)

sam = Agent(
    name="sam",
    instruction="You are sam, you need to figure out a solution based on the problem.",
)

if __name__ == "__main__":
    chain = Chain([john, sam])
    chain.runUntil("whats the best way to save engergy?", "consensus reached", debug=True)
